### geodsistance code for distance decay of beta diversity ####
# Distance decay of beta diversity

#function to calculate distance between points and print a matrix}
ReplaceLowerOrUpperTriangle <- function(m, triangle.to.replace){
  # If triangle.to.replace="lower", replaces the lower triangle of a square matrix with its upper triangle.
  # If triangle.to.replace="upper", replaces the upper triangle of a square matrix with its lower triangle.
  
  if (nrow(m) != ncol(m)) stop("Supplied matrix must be square.")
  if      (tolower(triangle.to.replace) == "lower") tri <- lower.tri(m)
  else if (tolower(triangle.to.replace) == "upper") tri <- upper.tri(m)
  else stop("triangle.to.replace must be set to 'lower' or 'upper'.")
  m[tri] <- t(m)[tri]
  return(m)
}

#calculate distance in meters between lat lon sites
GeoDistanceInMetresMatrix <- function(df.geopoints){
  # Returns a matrix (M) of distances between geographic points.
  # M[i,j] = M[j,i] = Distance between (df.geopoints$lat[i], df.geopoints$lon[i]) and
  # (df.geopoints$lat[j], df.geopoints$lon[j]).
  # The row and column names are given by df.geopoints$name.
  
  GeoDistanceInMetres <- function(g1, g2){
    # Returns a vector of distances. (But if g1$index > g2$index, returns zero.)
    # The 1st value in the returned vector is the distance between g1[[1]] and g2[[1]].
    # The 2nd value in the returned vector is the distance between g1[[2]] and g2[[2]]. Etc.
    # Each g1[[x]] or g2[[x]] must be a list with named elements "index", "lat" and "lon".
    # E.g. g1 <- list(list("index"=1, "lat"=12.1, "lon"=10.1), list("index"=3, "lat"=12.1, "lon"=13.2))
    DistM <- function(g1, g2){
      require("Imap")
      return(ifelse(g1$index > g2$index, 0, gdist(lat.1=g1$lat, lon.1=g1$lon, lat.2=g2$lat, lon.2=g2$lon, units="m")))
    }
    return(mapply(DistM, g1, g2))
  }
  
  n.geopoints <- nrow(df.geopoints)
  
  # The index column is used to ensure we only do calculations for the upper triangle of points
  df.geopoints$index <- 1:n.geopoints
  
  # Create a list of lists
  list.geopoints <- by(df.geopoints[,c("index", "lat", "lon")], 1:n.geopoints, function(x){return(list(x))})
  
  # Get a matrix of distances (in metres)
  mat.distances <- ReplaceLowerOrUpperTriangle(outer(list.geopoints, list.geopoints, GeoDistanceInMetres), "lower")
  
  # Set the row and column names
  rownames(mat.distances) <- df.geopoints$name
  colnames(mat.distances) <- df.geopoints$name
  
  return(mat.distances)
}

#categorizes each match as classified or unclassified
BOLDparser <- function(OTUtable) {
  for (i in 1:nrow(OTUtable)){
    if (is.na(OTUtable[i,c("Similarity")])) {
      OTUtable[i,c("Category")] = "Unclassified"
    }
    else if(OTUtable[i,c("Order")] == "Ixodida" & OTUtable[i,c("Similarity")] >= 84.4){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Order")] == "Mesostigmata" & OTUtable[i,c("Similarity")] >= 78.9){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Order")] == "Sarcoptiformes" & OTUtable[i,c("Similarity")] >= 74.9){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Order")] == "Trombidiformes" & OTUtable[i,c("Similarity")] >= 64.3){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Similarity")] >= 85){
      OTUtable[i,c("Category")] = "Classified"
    }
    else  {
      OTUtable[i,c("Category")] = "Unclassified"
    } 
  }
  return(OTUtable)
} # categorizes each OTU as classified or unclassified depending on specified thresholds


#assign taxa to higher groupings - only includes taxa in my file, some may be missing when used on other data and could be added
Phylum <- function(input) {
input$Kingdom[input$Phylum == "Amoebozoa"] <- "Protists"
input$Kingdom[input$Phylum == "Annelida"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Arthropoda"] <- "Arthropods"
input$Kingdom[input$Phylum == "Ascomycota"] <- "Fungi"
input$Kingdom[input$Phylum == "Bacillariophyta"] <- "Protists"
input$Kingdom[input$Phylum == "Basidiomycota"] <- "Fungi"
input$Kingdom[input$Phylum == "Chlorophyta"] <- "Protists"
input$Kingdom[input$Phylum == "Cnidaria"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Eukarya_unassigned"] <- "Protists"
input$Kingdom[input$Phylum == "Heterokontophyta"] <- "Protists"
input$Kingdom[input$Phylum == "Mollusca"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Nematoda"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Platyhelminthes"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Porifera"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Proteobacteria"] <- "Bacteria"
input$Kingdom[input$Phylum == "Rhodophyta"] <- "Protists"
input$Kingdom[input$Phylum == "Rotifera"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Tardigrada"] <- "Other invertebrates"
input$Kingdom[input$Phylum == "Zygomycota"] <- "Fungi"
input$Kingdom[input$Phylum == "Chordata"] <- "Chordates"
return (input)
}


ReplicateSlope <- function(LongAccum, SampleList) {
  samples <- as.data.frame(unique(SampleList$Sample))
  accumarth.slope <- data.frame() ## set up a blank data frame to add each selected match to
  for (i in 1:nrow(samples)){
    samp<- LongAccum[LongAccum$Grouping %in% samples[i,],] ## subsetting the matches for each sequence
    Sample <- samp[1, c("Grouping")]
    One  <- samp[samp$Obs =="1", "Richness"]
    Two  <- samp[samp$Obs =="2", "Richness"] - samp[samp$Obs =="1", "Richness"]
    Three  <- samp[samp$Obs =="3", "Richness"] - samp[samp$Obs =="2", "Richness"]
    Four  <- samp[samp$Obs =="4", "Richness"] - samp[samp$Obs =="3", "Richness"]
    Five  <- samp[samp$Obs =="5", "Richness"] - samp[samp$Obs =="4", "Richness"]
    Six  <- samp[samp$Obs =="6", "Richness"] - samp[samp$Obs =="5", "Richness"]
    Seven  <- samp[samp$Obs =="7", "Richness"] - samp[samp$Obs =="6", "Richness"]
    if (nrow(samp[samp$Obs =="8",]) == 1) {                           # one of the samples had a failed replicate so accounting for this
      Eight  <- samp[samp$Obs =="8", "Richness"] - samp[samp$Obs =="7", "Richness"]
    }
    else
      Eight  <- NA
    slope <- data.frame(Sample, One, Two, Three, Four, Five, Six, Seven, Eight)
    accumarth.slope<- rbind(accumarth.slope, slope) # selecting the closest match
  } 
  return(accumarth.slope)
}



#categorizes each match as classified or unclassified depending on specified thresholds (P95, trimmed thresholds from https://doi.org/10.1038/s41598-021-95147-8) for mites | JAMP thresholds for all else)
BOLDparser95 <- function(OTUtable) {
  for (i in 1:nrow(OTUtable)){
    if (is.na(OTUtable[i,c("Similarity")])) {
      OTUtable[i,c("Category")] = "Unclassified"
    }
    else if(OTUtable[i,c("Order")] == "Ixodida" & OTUtable[i,c("Similarity")] >= 84.4){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Order")] == "Mesostigmata" & OTUtable[i,c("Similarity")] >= 75.3){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Order")] == "Sarcoptiformes" & OTUtable[i,c("Similarity")] >= 74.9){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Order")] == "Trombidiformes" & OTUtable[i,c("Similarity")] >= 64.3){
      OTUtable[i,c("Category")] = "Classified"
    }
    else if(OTUtable[i,c("Similarity")] >= 85){
      OTUtable[i,c("Category")] = "Classified"
    }
    else  {
      OTUtable[i,c("Category")] = "Unclassified"
    } 
  }
  return(OTUtable)
} # categorizes each OTU as classified or unclassified depending on specified thresholds

#removes taxonomy below specified threshold (P95, trimmed from from https://doi.org/10.1038/s41598-021-95147-8 for mites | JAMP thresholds for all else)
BOLDparser295 <- function(OTUtable) { #yay it works! just need to have the thresholds in order of lowest to highest to make sure they are all applied appropriately
  for (i in 1:nrow(OTUtable)){
    if (OTUtable[i,c("Category")] == "Unclassified") {
      OTUtable[i,c("Phylum")] = "" ; OTUtable[i,c("Class")] = "" ; OTUtable[i,c("Order")] = ""
      ; OTUtable[i,c("Family")] = "" ; OTUtable[i,c("Genus")] = "" ; OTUtable[i,c("Species")] = ""
      ; OTUtable[i,c("Subspecies")] = ""
    }
    else if (OTUtable[i,c("Similarity")] < 64.3 & OTUtable[i,c("Order")] == "Trombidiformes" ) {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" 
      ; OTUtable[i,c("Genus")] = "" ; OTUtable[i,c("Family")] = "" # remove family names for sarcop below the sarcop threshold
    }
    else if (OTUtable[i,c("Similarity")] < 77.5 & OTUtable[i,c("Order")] == "Mesostigmata" ) {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" 
      ; OTUtable[i,c("Genus")] = "" ; OTUtable[i,c("Family")] = "" # remove family names for sarcop below the sarcop threshold
    }
    else if (OTUtable[i,c("Similarity")] < 83.1 & OTUtable[i,c("Order")] == "Sarcoptiformes" ) {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" 
      ; OTUtable[i,c("Genus")] = "" ; OTUtable[i,c("Family")] = "" # remove family names for sarcop below the sarcop threshold
    }
    else if (OTUtable[i,c("Similarity")] < 84.4 & OTUtable[i,c("Order")] == "Ixodida" ) {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" 
      ; OTUtable[i,c("Genus")] = "" ; OTUtable[i,c("Family")] = "" # remove family names for sarcop below the sarcop threshold
    }
        else if (OTUtable[i,c("Similarity")] < 90 & OTUtable[i,c("Order")] != "Sarcoptiformes" & OTUtable[i,c("Order")] != "Mesostigmata"
             & OTUtable[i,c("Order")] != "Trombidiformes" & OTUtable[i,c("Order")] != "Ixodida") {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" ; OTUtable[i,c("Genus")] = "" ; OTUtable[i,c("Family")] = "" # remove family names below 95%
    }
    else if (OTUtable[i,c("Similarity")] < 95) {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" ; OTUtable[i,c("Genus")] = "" # remove genus names below 95%
    }
    else if (OTUtable[i,c("Similarity")] < 98) {
      OTUtable[i,c("Subspecies")] = "" ; OTUtable[i,c("Species")] = "" # remove species and subspecies names below 98%
    }
  } 
  return(OTUtable)
} # removes taoxnomy  depending on specified thresholds


